/*
 *  ofxSndObj.h
 *  
 *
 *  Created by joshua noble on 10/4/08.
 *  thefactoryfactory
 *
 */

#ifndef OFX_SND_OBJ
#define OFX_SND_OBJ

#include "ofConstants.h"
#include "AudioDefs.h"
#include "ofMain.h"
#include "ofxBaseSndObj.h"
#include "ofxSOFilter.h"
#include "ofxSOTable.h"
#include "ofxSOOscilator.h"
#include "ofxSOMixer.h"
#include "ofxSOBuzz.h"
#include "ofxSOSpectrum.h"
#include "ofxSOPluck.h"
#include "ofxSOADSR.h"
#include "ofxSOWav.h"
#include "ofxSOLoop.h"
#include <map>
#include <vector>
#include <typeinfo>

class ofxBaseSndObj;
class ofxSOTable;
class ofxSOOscilator;
class ofxSOMixer;
class ofxSOBuzz;
class ofxSOSoundIn;
class ofxSOFilter;
class ofxSOSpectrum;
class ofxSOWav;
class ofxSOLoop;

enum {
	OFXSNDOBJBUZZ,
	OFXSNDOBJOSCILATOR,
	OFXSNDOBJSPECTRUM,
	OFXSNDOBJTABLE,
	OFXSNDOBJSOUNDIN,
	OFXSNDOBJMIXER,
	OFXSNDOBJBASE,
	OFXSNDOBJFILTER,
	OFXSNDOBJPLUCK,
	OFXSNDOBJADSR,
	OFXSNDOBJWAV,
	OFXSNDOBJLOOP
};


class ofxSndObj {
	
	public:
	
	float* left();
	float* right();
	
	ofxSndObj();
	~ofxSndObj();
	
	// basics
	int startSystemInOut(bool useSystemIn, bool useSystemOut);
	int startOut(bool useSystemOut, int device);
	
	bool hasInitedOut;
	bool hasInitedIn;
	
	// less advanced
	string createOscilator(ofxSOTable t, float frq, float amp);
	string createOscilator(ofxSOTable t, float frq, float amp, ofxBaseSndObj b, int type);
	string createBuzz(float fr, float amp, int harm);
	void removeOscilator(string key);
	void removeBuzz(string key);
	void tuneOscilator(string key, float frq, float amp);
	void tuneBuzz(string key, float frequency, float amp);
	
	string createNoise(float amp);
	void removeNoise(string key);
	void tuneNoise(string key, float amp);
	
	string createMixer();
	void addToMixer(string key, ofxBaseSndObj added, int type);
	void addToMixer(ofxSOMixer mix, ofxBaseSndObj added, int type);
	void removeFromMixer(string key, ofxBaseSndObj base, int type);
	
	string createPluck(float fr, float amp, float fdbgain, ofxBaseSndObj b, int type);
	void removePluck(string key);
	string createPluck(float fr, float amp, float fdbgain);
	void tunePluck(string key, float amp);
	void pluck(string key);
	
	string createLoop(float xfadetime, float looptime, ofxBaseSndObj inObject, int intype, float pitch);
	void resampleLoop(string key);
	void deleteLoop(string key);
	
	string createTable(long length, int type, float phase);
	void removeTable(string key);
	void tuneTable(string key, float frequency);
	
	string createFilter(string filterType, float frequency, float bandwidth, ofxBaseSndObj inObject,  int type);
	void removeFilter(string key);
	void tuneFilter(string key, float frequency, float bandwidth);
	
	string createSpecMult(string type, ofxBaseSndObj inObj1,  int obj1Type, ofxBaseSndObj inObj2, int obj2Type);
	void removeSpecMult(string key);
	
	int setOutput(ofxBaseSndObj outputObj, int intype, int channel);
	
	string loadSound(string filename, bool stereo, int channel);
	void deleteSound(string key);
	
	string createADSR(float attack, float maxamp, float decay, float sustain, float release, float duration);
	string createADSR(float attack, float maxamp, float decay, float sustain, float release, float duration, ofxBaseSndObj inObject, int type);
	void tuneADSRMaxAmp(string key, float maxamp);
	void tuneADSR(string key, float att, float dec, float sus, float rel);
	void tuneADSRDuration(string key, float dur);
	void adsrSustain(string key);
	
	int createFFT(ofxSOTable table, ofxBaseSndObj base, int type);
	
	// call this whenever you want to start the sound
	int startProcessing();
	int stopProcessing();
	
	// more advanced
	int addObject(SndObj* obj);
	int setOutput(SndObj* obj, int channel);
	
	void genFilter(string filterType, float frequency, float bandwidth, SndObj* inObject);
	string generateRandomStringKey();
	
	void cleanup();
	
protected:
		SndCoreAudio* core;
		SndRTIO* rtioIn;
		SndRTIO* rtioOut;
		SndThread thread;
		
		FFT* fft;
		
		SndBuffer* buffer;
		float* l;
		float* r;
		
		vector<SndObj*> addedObjs;
		
		SndObj* findSndObj(ofxBaseSndObj added, int type);
						   		   
		map<string, Oscili*> createdOscilis;
		map<string, Buzz*> createdBuzzes;
		map<string, SndLoop*> createdLoops;
		map<string, Rand*> createdNoises;
		map<string, Table*> createdTables;
		map<string, Mixer*> createdMixers;
		map<string, SndIn*> createdSndIns;
		map<string, Filter*> createdFilters;
		map<string, SpecMult*> createdSpectrums;
		map<string, Pluck*> createdPlucks;
		map<string, ADSR*> createdADSRs;
		map<string, SndWave*> createdSndWavs;
};

#endif